/*==============================================================================
File name:		GGSII_wave1_allcountries-ages
Task:			Deriving ages of household members from the GGS II, wave 1 datasets
				and counting members under certain ages
Author:			Carlotta Bochert
Written:		February 2023
==============================================================================*/

/*==============================================================================
					0. Summary
==============================================================================*/
/*
0.1 Purpose and organization of this do-file

	This do-file derives the survey date as well as birthdates and ages of
	respondent, partners, children and other household members from the month and
	year information given in the original dataset. It also computes respondents'
	ages at childbirth for each child.

	When specified missing values are derived from several missing inputs, the
	hierarchy is as follows:
		Not applicable (.c) taking precedence over
		Refusal (.b) taking precedence over
		Implausible values (.z) taking precedence over
		Incomplete survey (.h) taking precedence over
		Don't know (.a).
	For variables where these missing values are not differentiated, they are 
	subsumed under Missing (.m).
	
	In section 2:
		1. The survey date is transformed into a numeric variable.
		2. Respondents' date of birth (DOB) and age is computed.
		3. Respondents' children's DOB and age is computed, along with
			respondents' age at childbearing.
		4. Respondents' partner's DOB and age is computed.
		5. DOBs and ages for other household members are computed.
	
	All of these include suggestions for checking the plausibility of results. 
	Implausible values are not replaced automatically. Suggestions for how to do
	that are given in comments in the sections on respondents' and children's age.
	
	In section 3, variables are calculated to store whether there are
	household members below certain ages (6, 12, 16, 18, 25) and how many.
	
	In section 4, the resulting dataset is saved into a new file 
	([oldfilename]-ages.dta).
	
	For further information, please see the documentation in
	GGSII_wave1_allcountries-ages-readme.txt


0.2 Customize this do-file:
	In order to customize this Do-File (to replicate the results), please 
	1. enter the path of your working directory in 1.2. 
	2. check for correct specification of globals in 1.3 
		(create the respective folders, if necessary) 
	3. Choose the dataset you want to work on and define global dta accordingly
	
																			*/

/*==============================================================================
					1. Preparations: Configuring the do-file
==============================================================================*/

*1.1 Setup
version 14.0          // Stata version control (put your own version)
clear all             // clear working memory
macro drop _all       // clear macros

*1.2 Working directory
global wdir  "\\ia\NIDI$\home\CarlottaB\Documents\datawork"

*1.3 Define paths to subdirectories
global data 		"$wdir/0_data"   		// folder for original data
global posted 		"$wdir/1_posted"    	// folder for exported data

*Define which dataset to use (put in the file name without ending)

*global dta			"GGSII_Wave1_DK_V_1_0"			//Denmark
*global dta			"GGSII_Wave1_EE_V_1_0"			//Estonia
*global dta 		"GGSII_Wave1_UY_V_1_0"			//Uruguay
*global dta 			"GGP2020_WAVE1_MLD_V_0_2" 	//Moldova - old



use "$data/$dta.dta", clear		
*Keep only the variables needed for analysis (and respid for merging later)
	keep 																///
	respid intdatey intdatem											///
	dem02y dem02m dem22y dem22m lhi29_y* lhi29_m* hhd06_y* hhd06_m* 	///
	dem21 dem30a lhi31_* hhd01b


/*==============================================================================
					2. Computing dates and ages
==============================================================================*/
/*
For each type of person, there are two steps:

	1) Create date variables from month and year data. These count days from January
		1960 and assume that all events took place on the first of the month.
		
	2) Use this to calculate ages.
	*/

*2.1 Survey date
	*1)
		generate surveydate = dofm(ym(intdatey, intdatem))
		format surveydate %td
		label var surveydate "Date of survey"
		
		*check result
		tab surveydate intdatem, m 
	
		
*2.2 Age of respondent
	*1)
		generate dobr = dofm(ym(dem02y, dem02m))
		format dobr %td
		
		*replace with .a/.b/.z if month or year are unknown, refused or implausible
		replace dobr = .a if dem02m == .a | dem02y == .a
		replace dobr = .z if dem02m == .z | dem02y == .z | dem02y < 1000
		replace dobr = .b if dem02m == .b | dem02y == .b
		
		label var dobr "Respondent date of birth"
		*codebook dobr
		
	*2)
		generate ager = age(dobr, surveydate)
		*no additional missing values generated
		
		*replace with .a (Don't know) if the DOB is unknown
		replace ager = .a if dobr == .a
		*replace with .z (implausible) if the DOB is in the future or implausible
		replace ager = .z if (dobr > surveydate & dobr < .) | dobr == .z
		*replace with .b (Refusal) if the DOB is refused
		replace ager = .b if dobr == .b
		
		label var ager "Respondent age"
		
		*check that no respondent has a systemwide missing value on ager
		assert ager != .
			*If this results in an error message, check like this:
			*list if ager == .
			
		*Optional: screen respondents' ages in a table
			*tab ager, m
		
		
		/*If there are persons who are too old to be plausible (in this example,
		over 120), their ages can be set to .z like this: */
		*replace ager = .z if ager > 120 & ager < .
		
		*Optionally, those with implausible ages can also be dropped completely:
		*drop if ager == .z
		
		
*2.3 Age of partner
	*1)
		generate dobpartner = dofm(ym(dem22y, dem22m))
		format dobpartner %td
		
		*replace with .a/.b/.z if month or year are unknown, refused or implausible
		replace dobpartner = .a if dem22m == .a | dem22y == .a
		replace dobpartner = .z if dem22m == .z | dem22y == .z | dem22y < 1000
		replace dobpartner = .b if dem22m == .b | dem22y == .b | dem21 == .b
		*replace with .c (N.A.) if respondent has no partner
		replace dobpartner = .c if dem21 == 2
		
		label var dobpartner "Partner date of birth"
		codebook dobpartner
		
	*2)
		generate agepartner = age(dobpartner, surveydate)
			//missing values generated for those with DOBs in the future
		
		*replace with .a (Don't know) if the DOB is unknown
		replace agepartner = .a if dobpartner == .a
		*replace with .c (N.A.) if there is no partner
		replace agepartner = .c if dobpartner == .c
		*replace with .z (implausible) if the DOB is in the future or implausible
		replace agepartner = .z if (dobpartner > surveydate & dobpartner < .) ///
							| dobpartner == .z
		*replace with .b (Refusal) if the DOB is refused
		replace agepartner = .b if dobpartner == .b
		
		
		label var agepartner "Partner age"
		
		*check that no respondent has a systemwide missing value on agepartner
		assert agepartner != .
			*If this results in an error message, check like this:
			*list if agepartner == .
			
		*Optional: screen respondents' partners' ages in a table
		*tab agepartner, m
		
			/*Here, it might make sense to have a both a lower and an upper age
			boundary of plausibility.*/
		
	
*2.4 Age of children in the household
		*1)
		forvalues i = 1/20 {
			generate dobchild_`i'= dofm(ym(lhi29_y`i',lhi29_m`i'))
			format dobchild_`i' %td
			
			*replace with .a/.b/.z if month or year are unknown, refused or implausible
			replace dobchild_`i' = .a if lhi29_m`i' == .a | lhi29_y`i' == .a
			replace dobchild_`i' = .z if lhi29_m`i' == .z | lhi29_y`i' == .z ///
											| lhi29_y`i' < 1000
			replace dobchild_`i' = .b if lhi29_m`i' == .b | lhi29_y`i' == .b
			
			/*replace with .c (N.A.) if month and year were not answered, since 
			this is the case when the child does not exist*/
			replace dobchild_`i' = .c if lhi29_m`i' == . & lhi29_y`i' == .
			
			label var dobchild_`i' "Child `i' date of birth"
		}
		
		*2)			
		forvalues i = 1/20 {
			
			generate agechild_`i' = age(dobchild_`i', surveydate)
			//missing values generated for those with DOBs in the future
		
			*replace with .a (Don't know) if the DOB is unknown
			replace agechild_`i' = .a if dobchild_`i' == .a
			*replace with .c (N.A.) if the DOB is .c
			replace agechild_`i' = .c if dobchild_`i' == .c
			*replace with .z (implausible) if the DOB is in the future or implausible
			replace agechild_`i' = .z if dobchild_`i' > surveydate & dobchild_`i' < . ///
								| dobchild_`i' == .z
			*replace with .b (Refusal) if the DOB is refused
			replace agechild_`i' = .b if dobchild_`i' == .b
			
			label var agechild_`i' "Child `i' age"
		} 
		
		
		*check that no respondent has a systemwide missing value on any agechild*
		forvalues i = 1/20 {
			assert agechild_`i' != .
		}
		
		browse agechild* dobchild* lhi29_* if agechild_1 == .
			*If this results in an error message, check like this:
			/*
			forvalues i = 1/20 {
				list if agechild_`i' == .
			}
			*/
			
		*Optional: Check children's ages in a table
		/*	forvalues i = 1/20 {
				tab agechild_`i', m
			}
		*/
		
		/*
		/*If there are persons on any of the variables that are numbered
		consecutively (children, other household members) who are too old to be
		plausible (in this example, over 100), their ages can be set to .z like this: */
		forvalues i = 1/20 {
			replace agechild_`i' = .z if agechild_`i' > 100 & agechild_`i' < .
		}
		*/
		/*
		*Optionally, those with implausible ages can also be dropped completely:
		forvalues i = 1/20 {
			drop if agechild_`i' == .z
		}
		*/
		
		
	*Age of respondent at childbirth
		forvalues i = 1/20 {
			generate agebirth_`i' = age(dobr, dobchild_`i')
			
			*replace with .a (Don't know) if the DOB of respondent or child is unknown
			replace agebirth_`i' = .a if dobr == .a | dobchild_`i' == .a
			/*replace with .z (implausible) if either DOB is in the future or 
			implausible or child's dob is before respondent's */
			replace agebirth_`i' = .z if dobr == .z | dobchild_`i' == .z  			///
								| (dobchild_`i' > surveydate & dobchild_`i' < .)	///
								| (dobr > dobchild_`i' & dobr < .)
			*replace with .c (N.A.) if there is no child
			replace agebirth_`i' = .c if dobchild_`i' == .c
			*replace with .b (Refusal) if either DOB is refused
			replace agebirth_`i' = .b if dobr == .b | dobchild_`i' == .b
			
			label var agebirth_`i' "Age at birth of child `i'"
		} 
		
		
		
		*check that no respondent has a systemwide missing value on any agebirth*
		forvalues i = 1/20 {
			assert agebirth_`i' != .
		}
			*If this results in an error message, check like this:
			/*
			forvalues i = 1/20 {
				list if agebirth_`i' == .
			}
			*/
		*Optional: Check mothers' ages for plausibility
		/*	forvalues i = 1/20 {
				tab agebirth_`i' if dem01 == 2, m
			}
		*/		
		
*2.5 Age of other household members
		*1)
		forvalues i = 1/20 {
			generate dobother_`i'= dofm(ym(hhd06_y`i',hhd06_m`i'))
			format dobother_`i' %td
			/*replace with .a/.b/.z if month or year are unknown, refused or 
			implausible and .h if there was a break-off */
			replace dobother_`i' = .a if hhd06_m`i' == .a | hhd06_y`i' == .a
			replace dobother_`i' = .h if hhd06_m`i' == .h | hhd06_y`i' == .h ///
											| (hhd06_m`i' == . & hhd06_y`i' == . & hhd01b >= `i')
			replace dobother_`i' = .z if hhd06_m`i' == .z | hhd06_y`i' == .z ///
											| hhd06_y`i' < 1000
			replace dobother_`i' = .b if hhd06_m`i' == .b | hhd06_y`i' == .b
			
			/*replace with .c (N.A.) if there is no additional household member*/
			replace dobother_`i' = .c if hhd01b < `i'
			label var dobother_`i' "HH member `i' date of birth"
		}
		
		*Optional: Checking for birth dates after the survey date
		/*	forvalues i = 1/20 {
				tab dobother_`i' if dobother_`i' > surveydate & dobother_`i' != .
			}
		*/
	
		*2)	
		forvalues i = 1/20 {
			generate ageother_`i' = age(dobother_`i', surveydate)
			 
			*replace with .a (Don't know) if the DOB is unknown
			replace ageother_`i' = .a if dobother_`i' == .a
			*replace with .h (Don't know) if there was break-off before DOB
			replace ageother_`i' = .h if dobother_`i' == .h
			*replace with .c (N.A.) if this household member does not exist
			replace ageother_`i' = .c if dobother_`i' == .c
			*replace with .z (implausible) if the DOB is in the future
			replace ageother_`i' = .z if dobother_`i' == .z			///
								| (dobother_`i' > surveydate & dobother_`i' < .) 
			*replace with .b (Refusal) if the DOB is refused
			replace ageother_`i' = .b if dobother_`i' == .b
			
			label var ageother_`i' "HH member `i' age"
		}
		
		*check that no respondent has a systemwide missing value on any ageother*
		forvalues i = 1/20 {
			assert ageother_`i' != .
		}
			*If this results in an error message, check like this:
			/*
			forvalues i = 1/20 {
				list if ageother_`i' == .
			}
			*/
			
		*Optional: Check the resulting ages in a table
		/*	forvalues i = 1/20 {
				tab ageother_`i', m
			}
		*/

	
/*==============================================================================
					3. Counting household members under specific ages
==============================================================================*/
*Put in the values under which ages you want to count household members (here, 6, 12, 16, 18 and 25)

foreach j in 6 12 16 18 25 {

**3.1 Respondent
	*generate a helper variable that is 1 if the respondent is under `j' and 0 if over.
		gen hhru`j' = .
		replace hhru`j' = 1 if ager < `j'
		replace hhru`j' = .a if ager == .a
		replace hhru`j' = .z if ager == .z
		replace hhru`j' = .b if ager == .b
		replace hhru`j' = 0 if ager >= `j' & ager < .
		
		label var hhru`j' "Respondent under `j' [y/n]"
		
		label values hhru`j' lyn
		
		assert hhru`j' != .


**3.2 Partner
	
	/*generate helper var that is 1 if the household's partner is under `j'
	and 0 if there is no partner, the partner is over `j' or does not live in the household*/
		gen hhpu`j' = .
		replace hhpu`j' = 1 if dem30a == 1 & agepartner < `j'
		replace hhpu`j' = .a if dem30a == .a | agepartner == .a
		replace hhpu`j' = .z if dem30a == .z | agepartner == .z
		replace hhpu`j' = .b if dem30a == .b | agepartner == .b
		replace hhpu`j' = 0 if dem30a == 2 | (agepartner >= `j' & agepartner < .) | dem21 == 2
		
		label var hhpu`j' "Partner under `j' [y/n]"
		label values hhpu`j' lyn
	
	*Check there are no systemwide missing values on hhpu`j'
		assert hhpu`j' != .
	

**3.2 Children

	/*generate helper var that is 1 if the household's child is under `j'
	and 0 if the child is over `j' or lhi31* is not filled*/
		forval i = 1/20 {
			gen chu`j'_`i' = .
			replace chu`j'_`i' = 1 if inrange(lhi31_`i', 1, 3) & agechild_`i' < `j'
			replace chu`j'_`i' = .a if lhi31_`i' == .a | agechild_`i' == .a	
			replace chu`j'_`i' = .z if lhi31_`i' == .z | agechild_`i' == .z		
			replace chu`j'_`i' = .b if lhi31_`i' == .b | agechild_`i' == .b	
			replace chu`j'_`i' = 0 if lhi31_`i'== 4 | (agechild_`i' >= `j' & agechild_`i' < .)	
			replace chu`j'_`i' = 0 if lhi31_`i'== .	
			
			label var chu`j'_`i' "Child `i' under `j' [y/n]"
			label values chu`j'_`i' lyn
			
		}
		
		*check that no respondent has a systemwide missing value on any chu`j'*
		forvalues i = 1/20 {
			assert chu`j'_`i' != .
		}
	
	*Are there any?
	
		*generate helper variable to check for missing values on chu`j'_*
			egen tmpmishhchu`j' = rowmiss(chu`j'_*)
			
		*generate dummy variable for whether there are any household children under `j'
			egen hhanychu`j' = rowmax(chu`j'_*) 
			//ignores missing values, but returns .a if they are all missing
			
			*replace with missing if there are no positives and at least one is missing
			replace hhanychu`j' = .m if tmpmishhchu`j' > 0 & hhanychu`j' == 0	///
										| tmpmishhchu`j' == 20
			
			label var hhanychu`j' "Any HH children under `j' [y/n]"
	
	*Summing up household children under `j'
	
		*generate variable counting household children under `j'
			egen hhchu`j' = rowtotal(chu`j'_*)
				//this treats missings as 0
			
			*replace with missing if at least one is missing
			replace hhchu`j' = .m if tmpmishhchu`j' > 0
			
		label var hhchu`j' "Number of HH children under `j'"
	
	*Checks
		assert hhanychu`j' != .
		assert hhchu`j' != .
		
		
**3.3 Other household members	
	
	/*generate helper var that is 1 if the household member is under `j'
	and 0 if the household member is over `j' or this household member is not
	existent given the information in hhd01b*/
		forval i = 1/20 {
			gen otu`j'_`i' = .
			replace otu`j'_`i' = 1 if hhd01b >= `i'  & ageother_`i' < `j'
			replace otu`j'_`i' = .a if hhd01b >= `i' & ageother_`i' == .a	
			replace otu`j'_`i' = .h if hhd01b >= `i' & inlist(ageother_`i', .c, .h)
			replace otu`j'_`i' = .z if hhd01b >= `i' & ageother_`i' == .z		
			replace otu`j'_`i' = .b if hhd01b >= `i' & ageother_`i' == .b	
			replace otu`j'_`i' = 0 if hhd01b < `i' | (ageother_`i' >= `j' & ageother_`i' < .)	
			
			label var otu`j'_`i' "HH member `i' under `j' [y/n]"
			label values otu`j'_`i' lyn
		} 
		
		*check that no respondent has a systemwide missing value on any otu`j'*
		forvalues i = 1/20 {
			assert otu`j'_`i' != .
		}
	
	*Are there any?
	
		*generate helper variable to check for missing values on otu`j'*
			egen tmpmishhotu`j' = rowmiss(otu`j'_*)
			
		*generate dummy variable for whether there are any other household members under `j'
			egen hhanyotu`j' = rowmax(otu`j'_*) 
			//generally ignores missing values, but returns .a if they are all missing
			
			
			*replace with missing if there are no positives and at least one is missing
			replace hhanyotu`j' = .m if (tmpmishhotu`j' > 0 & hhanyotu`j' == 0)	///
										| tmpmishhotu`j' == 20
			
			label var hhanyotu`j' "Any other HH members under `j' [y/n]"
	
	*Summing up household children under `j'
	
		*generate variable counting other household members under `j'
			egen hhotu`j' = rowtotal(otu`j'_*)
				//this treats missings as 0
			
			*replace with missing if at least one is missing
			replace hhotu`j' = .m if tmpmishhotu`j' > 0
			
		label var hhotu`j' "Number of other HH members under `j'"
	
	*Checks
		assert hhanyotu`j' != .
		assert hhotu`j' != .
	


*3.4 Count all household members under `j'

  *Are there any?
	*generate dummy variable for whether there are any household members under `j'
	egen hhanyu`j' = rowmax(hhru`j' hhpu`j' hhanychu`j' hhanyotu`j')
	//generally ignores missing values, but returns .a if they are all missing	
		
	*generate helper variable to check for missing values on the contributing variables
		egen tmpmishhanyu`j' = rowmiss(hhru`j' hhpu`j' hhanychu`j' hhanyotu`j')
		
	*replace with missing if none are positive and at least one is missing
		replace hhanyu`j' = .m if (tmpmishhanyu`j' > 0 & hhanyu`j' == 0)	///
								| tmpmishhanyu`j' == 20
			

  *Summing up all household members under `j'
	gen tothhu`j' = hhru`j' + hhpu`j' + hhchu`j' + hhotu`j'
	
		*generate helper variable to check for missing values on the contributing variables
			egen tmpmishhu`j' = rowmiss(hhru`j' hhpu`j' hhchu`j' hhotu`j')
			
		*replace with missing if at least one is missing
			replace tothhu`j' = .m if tmpmishhu`j' > 0
	
	label var tothhu`j' "Number of HH members under `j'"
	
	assert hhanyu`j' != .
	assert tothhu`j' != .
	
}		
	

	
/*==============================================================================
					4. Export dataset
==============================================================================*/		
*Drop temporary variables
	drop tmpmis*

*Keep only the newly created variables (and respid for merging later)
	keep 	respid															///
	surveydate dobr ager dobpartner agepartner dobchild_* agechild_* 		///
	agebirth_* dobother_* ageother_* 										///
	hhru* hhpu* chu* otu* hhanychu* hhchu* hhanyotu* hhotu* hhanyu* tothhu*


*Label missing values for all variables

	* Add value labels to those variables without
		findname, type(numeric byte int long float double) local(Notstring)
		findname, vallabel not local(Nolab)
		foreach v of local Notstring {
			if `: list b in Nolab' {
				lab def `v', add
				lab val `v' `v'
			}
		}

	* Add consistent labels for missing values
		quietly label dir
		foreach lab in `r(names)' {
			lab def `lab' 	.a "Does not know" .b "Refusal" 				///
							.c "Not applicable/no response/reporting error" ///
							.h "Incomplete survey" .m "Missing"  			///
							.z "Implausible value", modify
		}	
	*Note: This also gives a value label to respid and modifies it.
	
*Save dataset in posted
	save "$posted/${dta}-ages.dta", replace
	
	